@@ -137,13 +137,20 @@ DefaultAction.prototype.resumeEvent = function() {
 };
 
 DefaultAction.prototype.mouseMoveEvent = function(event) {
-    // we're in the middle of panning: don't do anything:
+    // we're in the middle of panning:
+    // do nothing:
     if (event.buttons().valueOf() === Qt.MidButton.valueOf() ||
         (event.buttons().valueOf() === Qt.LeftButton.valueOf() &&
          event.modifiers().valueOf() === Qt.ControlModifier.valueOf())) {
         return;
     }
 
+    // we're in the middle of choosing something from a context menu:
+    // do nothing:
+    if (this.waitingForContextMenu) {
+        return;
+    }
+
     var view, referencePoint, entityId, range;
     
     if (isNull(this.pickRangePixels)) {
@@ -178,7 +185,8 @@ DefaultAction.prototype.mouseMoveEvent = function(event) {
             // if the dragging started on top of a reference point,
             // start moving the reference point:
             referencePoint = view.getClosestReferencePoint(this.d1Screen, this.minPickRangePixels);
-            if (referencePoint.isValid() && !referencePoint.isIgnore() && !RSettings.getIgnoreAllReferencePoints()) {
+            if (referencePoint.isValid() && !referencePoint.isIgnore() && !RSettings.getIgnoreAllReferencePoints() &&
+                !referencePoint.isArrow()) {
                 this.d1Model = referencePoint;
                 this.di.setRelativeZero(this.d1Model);
                 this.setState(DefaultAction.State.MovingReference);
@@ -274,11 +282,16 @@ DefaultAction.prototype.mouseMoveEvent = function(event) {
 };
 
 DefaultAction.prototype.mouseReleaseEvent = function(event) {
+    if (this.waitingForContextMenu) {
+        return;
+    }
+
     var persistentSelection = RSettings.getBoolValue("GraphicsView/PersistentSelection", false);
     var entityId;
 
-    var shiftPressed = this.isShiftPressed(event) || (event.modifiers() & Qt.ControlModifier) > 0 || persistentSelection===true;
-    var addToSelection = this.getAddToSelection(shiftPressed);
+    var shiftPressed = isShiftPressed(event);
+    var controlPressed = isControlPressed(event);
+    var addToSelection = this.getAddToSelection(shiftPressed) || controlPressed || persistentSelection===true;
 
     if (event.button() === Qt.LeftButton) {
         switch (this.state) {
@@ -300,7 +313,6 @@ DefaultAction.prototype.mouseReleaseEvent = function(event) {
                 }
             }
 
-
             // find selectable entity under cursor:
             entityId = this.getEntityId(event, false, true);
 
@@ -329,17 +341,9 @@ DefaultAction.prototype.mouseReleaseEvent = function(event) {
             this.di.clearPreview();
             this.d2Model = event.getModelPosition();
 
-            if (addToSelection ||
-                (event.modifiers().valueOf() == Qt.ControlModifier.valueOf()) ||
-                persistentSelection===true) {
+            // select entities:
+            this.di.selectBoxXY(new RBox(this.d1Model, this.d2Model), addToSelection);
 
-                // add all entities in window to the selection:
-                this.di.selectBoxXY(new RBox(this.d1Model, this.d2Model), true);
-            } else {
-                // select entities in window only:
-                this.di.selectBoxXY(new RBox(this.d1Model, this.d2Model), false);
-            }
-            // event.setConsumed(true);
             this.setState(DefaultAction.State.Neutral);
             break;
 
@@ -454,7 +458,7 @@ DefaultAction.prototype.escapeEvent = function(event) {
 DefaultAction.prototype.pickCoordinate = function(event, preview) {
     var op;
 
-    var shiftPressed = this.isShiftPressed(event);
+    var shiftPressed = isShiftPressed(event);
     var addToSelection = this.getAddToSelection(shiftPressed);
     var doc = this.getDocument();
 
@@ -669,7 +673,7 @@ DefaultAction.prototype.entityDoubleClicked = function(entityId, event) {
             // block editing with double click might be disabled for this entity:
             if (entity.getCustomBoolProperty("QCAD", "DoubleClickEditBlock", true)===true) {
                 include("scripts/Block/Block.js");
-                EAction.handleUserMessage(qsTr("Editing block '%1'<br>Choose <i>Block > Return to Main Drawing</i> when done").arg(entity.getReferencedBlockName()), false);
+                EAction.handleUserMessage(qsTr("Editing block \"%1\"<br>Choose <i>Block > Return to Main Drawing</i> when done").arg(entity.getReferencedBlockName()), false);
                 Block.editBlock(this.di, entity.getReferencedBlockName());
             }
         }
@@ -689,7 +693,7 @@ DefaultAction.prototype.entityDoubleClicked = function(entityId, event) {
         if (RSettings.getBoolValue("GraphicsView/DoubleClickSelectContour", true)===true) {
             include("scripts/Select/SelectContour/SelectContour.js");
             var matchingEntityIds = SelectContour.getConnectedEntities(this.document, entityId, 0.001);
-            var add = this.isShiftPressed(event);
+            var add = isShiftPressed(event);
             if (entity.isSelected()) {
                 this.di.selectEntities(matchingEntityIds, add);
             }
@@ -705,19 +709,6 @@ DefaultAction.prototype.entityDoubleClicked = function(entityId, event) {
 //    }
 };
 
-/**
- * \return True if the shift key is pressed to alter the function (e.g. add to selection).
- */
-DefaultAction.prototype.isShiftPressed = function(event) {
-    if (isFunction(event.getModifiers)) {
-        return (event.getModifiers() & Qt.ShiftModifier) > 0;
-    }
-    if (isFunction(event.modifiers)) {
-        return (event.modifiers().valueOf() === Qt.ShiftModifier.valueOf());
-    }
-    return false;
-};
-
 /**
  * \return True to add to selection (by default this is when shiftPressed is true).
  * May be reimplemented to allow other ways to add to selection (e.g. user interface button, etc.)
